@@ -1,68 +1,172 @@
 #!/usr/bin/env RScript
 #contributors=c("Gregory Smith", "Nils Jenke", "Michael Gruenstaeudl")
 #email="m_gruenstaeudl@fhsu.edu"
-#version="2024.01.07.2200"
+#version="2024.02.01.1736"
 
-read.gb2DF <- function(gbkData) {
+read.gb2DF <- function(gbkData, IRPresenceAndSyntenyCheck) {
   fileDF <- data.frame()
   for (sample in gbkData) {
-    sampleDF <- parseFeatures(sample$FEATURES)
-    fileDF <- dplyr::bind_rows(fileDF, sampleDF)
+    sampleDF <- parseFeatures(sample$FEATURES, IRPresenceAndSyntenyCheck)
+    if (!is.null(sampleDF)) {
+      fileDF <- dplyr::bind_rows(fileDF, sampleDF)
+    }
+  }
+  # signal no usable data
+  if (ncol(fileDF) == 0 && nrow(fileDF) == 0) {
+    return(NULL)
   }
   return(fileDF)
 }
 
-parseFeatures <- function(features) {
+parseFeatures <- function(features, IRPresenceAndSyntenyCheck) {
   sampleDF <- data.frame()
   for (feature in features) {
     feature <- parseFeature(feature)
-    sampleDF <- dplyr::bind_rows(sampleDF, feature)
+    if (!is.null(feature)) {
+      sampleDF <- dplyr::bind_rows(sampleDF, feature)
+    }
   }
-  type <- NULL
-  source <- sampleDF %>%
-              dplyr::filter(type=="source")
-  sampleDF <- sampleDF %>% 
-                dplyr::mutate(seqnames = as.factor(source[, "organism"])) %>%
-                dplyr::select(dplyr::all_of(c("seqnames", "start", "end", 
-                                              "gene", "note", "standard_name", 
-                                              "type")))
+  # check if can we can use the sample
+  subsetCols <- checkFeatureQualifiers(sampleDF, IRPresenceAndSyntenyCheck)
+  if (is.null(subsetCols)) {
+    return(NULL)
+  }
+  rownames(sampleDF) <- NULL
+  
+  # create derivative "start" and "end" variables from "locations",
+  # create "seqname", and subset according to analysis needs
+  . <- NULL
+  sampleDF <- sampleDF %>%
+                addStartEnd() %>%
+                addSeqname() %>%
+                subset4Analysis(., subsetCols)
   return(sampleDF)
 }
 
 parseFeature <- function(feature) {
-  # transform source data frame
+  # first check if feature is desired
+  locAndTypeIndex <- 1
+  type <- feature[locAndTypeIndex,1]
+  if (isIgnoredFeature(type)) {
+    return(NULL)
+  }
+  
+  # transform source data frame making sure final result
+  # is still data frame
   feature <- as.data.frame(t(feature))
-  colnames(feature) <- feature[1,]
-  feature <- feature[-1,]
-  rownames(feature) <- NULL
+  colNames <- feature[1,]
+  feature <- as.data.frame(feature[-1,])
+  colnames(feature) <- colNames
   
   # fix sequence location(s) and feature type
-  locAndTypeIndex <- 1
-  locationsStr <- feature[1,locAndTypeIndex]
-  type <- names(feature)[locAndTypeIndex]
   feature <- feature %>% 
               dplyr::rename_with(~ "locations", 
                                  .cols = dplyr::all_of(locAndTypeIndex)) %>%
               dplyr::mutate(type = type)
-  
-  # create derivative "start" and "end" variables from "locations"
-  # and create feature copies for multiple sequences
-  locationsList <- 
-    regmatches(locationsStr, gregexpr("\\d+", locationsStr))[[1]] %>%
-    as.integer()
-  feature <- feature %>%
-              dplyr::mutate(start = NA, end = NA)
-  for (pairIndex in 1:(length(locationsList) / 2)) {
-    if (pairIndex > 1) {
-      feature <- dplyr::bind_rows(feature, feature[pairIndex-1,])
-    }
-    startIndex = pairIndex * 2 - 1
-    feature[pairIndex, "start"] = locationsList[startIndex]
-    feature[pairIndex, "end"]   = locationsList[startIndex+1]
-  }
   return(feature)
 }
 
+addStartEnd <- function(sampleDF) {
+  start <-
+    end <- 
+    feature_index <-
+    . <-
+    NULL
+  
+  # keep track of feature ordering in case of multiple sequences
+  # for features
+  sampleDF <- sampleDF %>%
+                dplyr::mutate(feature_index = as.integer(rownames(.)))
+  
+  locationsList <- getLocationsList(sampleDF)
+  for (index in 1:length(locationsList)) {
+    featureLocations <- locationsList[[index]]
+    locationsCount <- length(featureLocations)
+    if (locationsCount == 0) {
+      next
+    }
+    
+    sampleDF[index, c("start", "end")] <- c(featureLocations[[1]][[1]],
+                                            featureLocations[[1]][[2]])
+    # create feature copies for multiple sequences
+    if (locationsCount > 1) {
+      sampleDF <- addMultiSeqFeats(sampleDF, featureLocations, index)
+    }
+  }
+  
+  sampleDF <- sampleDF %>%
+                dplyr::mutate(start = as.integer(start),
+                              end = as.integer(end)) %>%
+                dplyr::arrange(feature_index) %>%
+                dplyr::select(-feature_index)
+  return(sampleDF)
+}
+
+addMultiSeqFeats <- function(sampleDF, featureLocations, index) {
+  . <- NULL
+
+  featureLocations <- featureLocations[-1]
+  featureCopies <- sampleDF %>%
+    dplyr::slice(index) %>%
+    replicate(length(featureLocations),
+              ., simplify=FALSE) %>%
+    dplyr::bind_rows() %>%
+    dplyr::mutate(start = featureLocations[[1]][[1]],
+                  end = featureLocations[[1]][[2]])
+  sampleDF <- sampleDF %>%
+    dplyr::bind_rows(., featureCopies)
+  return(sampleDF)
+}
+
+addSeqname <- function(sampleDF) {
+  type <- NULL
+  source <- sampleDF %>%
+    dplyr::filter(type=="source")
+  sampleDF <- sampleDF %>%
+    dplyr::mutate(seqnames = as.factor(source[, "organism"]))
+  return(sampleDF)
+}
+
+subset4Analysis <- function(sampleDF, subsetCols) {
+  sampleDF <- sampleDF %>%
+    dplyr::select(dplyr::all_of(subsetCols))
+  return(sampleDF)
+}
+
+getLocationsList <- function(sampleDF, removeRemotes=TRUE) {
+  locationsList <- sampleDF %>%
+                    dplyr::pull("locations") %>%
+                    lapply(normalizeLocation) %>%
+                    lapply(function(x) unlist(strsplit(x, ",")))
+  if (removeRemotes) {
+    locationsList <- lapply(locationsList,
+                            function(x) x[!grepl(":", x)])
+  }
+  locationsList <- lapply(locationsList, 
+                          function(x) strsplit(x, ":|\\.{2,}"))
+  return(locationsList)
+}
+
+# based on standard listed by INSDC
+# https://www.insdc.org/submitting-standards/feature-table/
+normalizeLocation <- function(locationsStr) {
+  # standardize delimiter
+  cleanStr <- gsub("(?![A-Za-z]+)(\\d+)\\.(\\d+)(?!:)", 
+                   "\\1..\\2", locationsStr, perl=TRUE)
+  cleanStr <- gsub("(?![A-Za-z]+)(\\d+)\\^(\\d+)(?!:)", 
+                   "\\1..\\2", cleanStr, perl=TRUE)
+  # remove unknown boundary identifiers
+  cleanStr <- gsub("[<>](\\d+)", 
+                   "\\1", cleanStr)
+  # remove operators
+  cleanStr <- gsub("complement|join|order|[\\(\\)]", 
+                   "", cleanStr)
+  # standardize single bases
+  cleanStr <- gsub("(^|,)(.+:)?(\\d+)($|,)", 
+                   "\\1\\2\\3..\\3\\4", cleanStr, perl=TRUE)
+  return(cleanStr)
+}
+
 read.gbSeq <- function(gbkData) {
   sampleSequences <- c()
   for (sample in gbkData) {
@@ -73,20 +177,22 @@ read.gbSeq <- function(gbkData) {
 
 read.gbGenes <- function(gbkDataDF) {
   type <- NULL
+  subsetCols <- c("seqnames", "start", "end", "gene")
   gene_L <- gbkDataDF %>%
               dplyr::filter(type=="gene") %>% 
-              dplyr::select(dplyr::all_of(c("seqnames", "start", "end", "gene")))
+              dplyr::select(dplyr::all_of(subsetCols))
   rownames(gene_L) <- NULL
   return(gene_L)
 }
 
 read.gbOther <- function(gbkDataDF) {
   type <- NULL
+  subsetCols <- c("seqnames", "start", "end", 
+                  "gene", "note", "standard_name")
   regions <- gbkDataDF %>%
               dplyr::filter(!type %in% c("gene", "exon", "transcript",
                                   "CDS", "variant")) %>% 
-              dplyr::select(dplyr::all_of(c("seqnames", "start", "end", 
-                              "gene", "note", "standard_name")))
+              dplyr::select(dplyr::all_of(subsetCols))
   rownames(regions) <- NULL
   return(regions)
 }
@@ -103,8 +209,8 @@ read.gbSampleName <- function(gbkData) {
   sampleNames <- c()
   for (sample in gbkData) {
     sampleNames <- c(sampleNames, 
-                   c(sample_name = sample$ACCESSION, 
-                     genome_name = sample$VERSION))
+                   c(sample_name = sample$VERSION,
+                     genome_name = sample$ACCESSION))
   }
   return(sampleNames)
 }
@@ -265,3 +371,27 @@ validateColors <- function(colorsToValidate) {
     stop("Unsupported R plot color defined.")
   }
 }
+
+checkFeatureQualifiers <- function(sampleDF, IRPresenceAndSyntenyCheck) {
+  subsetCols <- c("gene", "note", "type")
+  if (IRPresenceAndSyntenyCheck) {
+    subsetCols <- c(subsetCols, "standard_name")
+  }
+  missingCols <- subsetCols[!(subsetCols %in% colnames(sampleDF))]
+  if (length(missingCols) > 0) {
+    logger::log_warn(paste0("Unable to analyze sample as specified; ", 
+                            "missing feature qualifiers: ",
+                            "'",
+                            paste(missingCols, collapse = "', '"),
+                            "'"))
+    return(NULL)
+  }
+  # add future generated columns that are needed
+  subsetCols <- c(subsetCols, "start", "end", "seqnames")
+  return(subsetCols)
+}
+
+isIgnoredFeature <- function(featureName) {
+  ignoredFeatures <- c("D-loop")
+  return(featureName %in% ignoredFeatures)
+}
